//*****************************************************************************
//  MSP430G2131 / DRV8837C Customer EVM Demo - Syncronous Rectification
//  
//  Description: This program operates MSP430 normally in LPM0 with WDT ISR 
//  used for system wake-up. Using DRV8837C Customer EVM PWM_SEL_IN open selects
//  forward, closed selects reverse direction. Timer_A CCR1 drives in upmode, 
//  5-bit PWM. PWM DutyCycle is measured using poti on ADC10_A0.  
//  LED is toggled in while()speed relative counting up LEDcounter.
//  Forward Synchronous is defined as; IN1 = 1, IN2 modulated 
//  Reverse Synchronous is defined as; IN2 = 1, IN1 modulated  
//  ACLK = n/a, MCLK = TACLK = SMCLK = 1MHz calibrated DCO
//  PWM frequency = TACLK/32 = 31.250Khz  
//
//  M. Buccini
//  July 27, 2012
//
// * Copyright (c) 2016, Texas Instruments Incorporated
// * All rights reserved.
// *
// * Redistribution and use in source and binary forms, with or without
// * modification, are permitted provided that the following conditions
// * are met:
// *
// * *  Redistributions of source code must retain the above copyright
// *    notice, this list of conditions and the following disclaimer.
// *
// * *  Redistributions in binary form must reproduce the above copyright
// *    notice, this list of conditions and the following disclaimer in the
// *    documentation and/or other materials provided with the distribution.
// *
// * *  Neither the name of Texas Instruments Incorporated nor the names of
// *    its contributors may be used to endorse or promote products derived
// *    from this software without specific prior written permission.
// *
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//******************************************************************************

#include "msp430g2131.h"

#define LED         BIT1                // P1.1
#define IN_1_PWM    BIT2                // P1.2
#define PWM_SEL_IN  BIT3                // P1.3
#define ADC_VREF    BIT4                // P1.4
#define IN_2_PWM    BIT6                // P2.6

unsigned int LEDcounter = 0;            // Countdown timer for LED toggle
unsigned int DutyCycle;                 // 6-bit PWM duty cycle

int main(void)
{
  WDTCTL = WDT_MDLY_8;                  // Set Watchdog Timer interval to ~30ms
  IE1 |= WDTIE;                         // Enable WDT interrupt
  BCSCTL1 = CALBC1_1MHZ;                // Set DCO to 1MHz
  DCOCTL = CALDCO_1MHZ;
  P1OUT = 0x00;                         // P1.x Reset
  P1DIR = 0xF6;                         // P1.3,0 inputs, else outputs
  P2OUT = 0x00;                         // P2.x reset
  P2SEL = 0x00;                         // P2.x no options
  P2DIR = 0xFF;                         // P2.x outputs
  CCTL1 = OUTMOD_7;                     // CCR1 in set/reset mode
  CCR1 = 0;				// CCR1 Duty cycle to 0%
  CCR0 = 64 - 1;                        // 6-bit PWM period
  TACTL = TASSEL_2 + MC_1;              // SCLK source, up mode  
  ADC10CTL0 = ADC10SHT_2 + ADC10ON;     // VCC ref
  ADC10AE0 |= INCH_0;                   // P1.0 ADC option select    

  while (1){
    _BIS_SR(LPM0_bits + GIE);           // Enter LPM0 with interrupts
    P1OUT |= ADC_VREF;                  // Enable poti divider    
    ADC10CTL0 |= ENC + ADC10SC;         // Sampling and conversion start
    while (ADC10IFG & ADC10CTL0);       // Wait for conversion complete 
    ADC10CTL0 &= ~ADC10IFG;             // Clear converstion flag
    P1OUT &= ~ADC_VREF;                 // Disable poti divider    
    DutyCycle = 63 - (ADC10MEM>>4);     // Mask upper 6-bits
    CCR1 = DutyCycle;                   // PWM duty cycle to DRV

    if (DutyCycle == 63) {              // Coast? 
      P1OUT &= ~IN_1_PWM;               // yes, IN_1 reset 
      P2OUT &= ~IN_2_PWM;               // IN_2 reset
      P1SEL = 0x00;                     // Disable IN_1 PWM
      P2SEL = 0x00;                     // Disable IN_2 PWM
    }
    else if (PWM_SEL_IN & P1IN) {       // Forward direction? 
      P1OUT |= IN_1_PWM;                // yes, IN_1 set 
      P1SEL = 0x00;                     // Disable IN_1 PWM
      P2SEL = IN_2_PWM;                 // Enable IN_2 PWM 
    }
    else {                              // Reverse 
      P2OUT |= IN_2_PWM;                // IN_2 set
      P2SEL = 0x00;                     // Disable IN_2 PWM
      P1SEL = IN_1_PWM;                 // Enable IN_1 PWM 
    }
    LEDcounter++;
    if(LEDcounter > DutyCycle){
      P1OUT ^= LED;                     // Toggle LED
      LEDcounter = 0;                   // Reset LED Counter
    }
  }
}
// Watchdog Timer interrupt service routine
#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer(void)
{
  _BIC_SR_IRQ(LPM3_bits);                 // Clear LPM0 bits from 0(SR)
}
  
  
